@@ -1,13 +1,12 @@
-# $Id: /mirror/gungho/lib/Gungho/Base.pm 6450 2007-04-10T01:52:17.416998Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Base.pm 8892 2007-11-10T14:11:01.888849Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
 
 package Gungho::Base;
 use strict;
-use base qw(Class::Accessor::Fast);
-use Class::C3;
-INIT { Class::C3::initialize() }
+use warnings;
+use base qw(Gungho::Base::Class Class::Accessor::Fast);
 
 __PACKAGE__->mk_accessors($_) for qw(config);
 
@@ -15,12 +14,9 @@ sub new
 {
     my $class  = shift;
     my $self = bless { @_ }, $class;
-    $self->config({}) unless $self->config;
     return $self;
 }
 
-sub setup {}
-
 1;
 
 __END__
