@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 8884 2007-11-09T11:12:49.102360Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 31116 2007-11-26T13:09:51.933010Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -13,6 +13,7 @@ use POE::Component::Client::HTTP;
 
 __PACKAGE__->mk_accessors($_) for qw(alias loop_alarm loop_delay resolver clients);
 
+use constant DEBUG => 0;
 use constant UserAgentAlias => 'Gungho_Engine_POE_UserAgent_Alias';
 use constant DnsResolverAlias => 'Gungho_Engine_POE_DnsResolver_Alias';
 use constant SKIP_DECODE_CONTENT  =>
@@ -134,7 +135,7 @@ sub run
                 _start => '_poe_session_start',
                 _stop  => '_poe_session_stop',
                 map { ($_ => "_poe_$_") }
-                    qw(session_loop start_request handle_response got_dns_response)
+                    qw(session_loop start_request handle_response got_dns_response shutdown)
             }
         ]
     );
@@ -142,6 +143,23 @@ sub run
     POE::Kernel->run();
 }
 
+sub stop
+{
+    my ($self, $c) = @_;
+    POE::Kernel->post($self->alias, 'shutdown');
+}
+
+sub _poe_shutdown
+{
+    my ($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
+    my $clients = $self->clients;
+    foreach my $client (@$clients) {
+        $kernel->post($client, 'shutdown');
+    }
+    my $c = $heap->{CONTEXT};
+    $c->is_running(0);
+}
+
 sub _poe_session_start
 {
     $_[KERNEL]->alias_set( $_[OBJECT]->alias );
@@ -180,13 +198,14 @@ sub _poe_session_loop
     }
     $self->loop_alarm($kernel->alarm_set('session_loop', time() + $delay));
 
-    $c->run_hook('engine.end_loop');
+    $c->notify('engine.end_loop');
 }
 
 sub send_request
 {
     my ($self, $c, $request) = @_;
     POE::Kernel->post($self->alias, 'start_request', $request);
+    return 1;
 }
 
 sub _poe_start_request
@@ -216,9 +235,9 @@ sub _poe_start_request
         return;
     }
 
-    $c->run_hook('engine.send_request', { request => $request });
+    $c->notify('engine.send_request', { request => $request });
 
-    { # XXX - for debug only...
+    if (DEBUG) {
         my $uri = $request->uri->clone;
         $uri->host( $request->notes('original_host') ) if $request->notes('original_host');
         $c->log->info("Going to fetch $uri");
@@ -254,7 +273,7 @@ sub _poe_handle_response
         # Put it back
         $req->uri->host($host);
     }
-    { # XXX - for debug only...
+    if (DEBUG) {
         $c->log->info("Received " . $req->uri);
     }
 
@@ -276,7 +295,7 @@ sub _poe_handle_response
         }
     }
 
-    $c->run_hook('engine.handle_response', { request => $req, response => $res });
+    $c->notify('engine.handle_response', { request => $req, response => $res });
 
     # Do we support auth challenge ?
     my $code = $c->can('check_authentication_challenge');
@@ -440,6 +459,10 @@ sets up the engine.
 Instantiates a PoCo::Client::HTTP session and a main session that handles the
 main control.
 
+=head2 stop
+
+Shutsdown the engine
+
 =head2 send_request($request)
 
 Sends a request to the http client
